import Protocol from 'devtools-protocol';
import { TBBaseCommand, flags } from '../../command-config';
import { IMarker } from '@tracerbench/core';
import { ITBConfig } from '../../command-config/tb-config';
import { IConfig } from '@oclif/config';
export interface ICompareFlags {
    hideAnalysis: boolean;
    browserArgs: string[];
    cpuThrottleRate: number;
    fidelity: number;
    markers: IMarker[];
    network: Protocol.Network.EmulateNetworkConditionsRequest;
    tbResultsFolder: string;
    controlURL: string | undefined;
    experimentURL: string | undefined;
    tracingLocationSearch: string;
    runtimeStats: boolean;
    emulateDevice?: string;
    emulateDeviceOrientation?: string;
    socksPorts?: [string, string] | [number, number] | undefined;
    debug: boolean;
    regressionThreshold?: number;
    headless: boolean;
    config?: string;
    report?: boolean;
}
export default class Compare extends TBBaseCommand {
    static description: string;
    static flags: {
        hideAnalysis: import("@oclif/parser/lib/flags").IBooleanFlag<boolean>;
        browserArgs: flags.IOptionFlag<any>;
        cpuThrottleRate: flags.IOptionFlag<any>;
        fidelity: flags.IOptionFlag<any>;
        markers: flags.IOptionFlag<any>;
        network: flags.IOptionFlag<any>;
        tbResultsFolder: flags.IOptionFlag<string>;
        controlURL: flags.IOptionFlag<string | undefined>;
        experimentURL: flags.IOptionFlag<string | undefined>;
        tracingLocationSearch: flags.IOptionFlag<string>;
        emulateDevice: flags.IOptionFlag<string | undefined>;
        emulateDeviceOrientation: flags.IOptionFlag<string | undefined>;
        socksPorts: flags.IOptionFlag<any>;
        regressionThreshold: flags.IOptionFlag<any>;
        config: flags.IOptionFlag<string | undefined>;
        runtimeStats: import("@oclif/parser/lib/flags").IBooleanFlag<boolean>;
        report: import("@oclif/parser/lib/flags").IBooleanFlag<boolean>;
        debug: import("@oclif/parser/lib/flags").IBooleanFlag<boolean>;
        headless: import("@oclif/parser/lib/flags").IBooleanFlag<boolean>;
    };
    compareFlags: ICompareFlags;
    parsedConfig: ITBConfig;
    explicitFlags: string[];
    analyzedJSONString: string;
    constructor(argv: string[], config: IConfig);
    init(): Promise<void>;
    run(): Promise<string>;
    private parseFlags;
    /**
     * Final result of the configs are in the following order:
     *
     * controlConfigs = tbconfig:controlBenchmarkEnvironment > command line > tbconfig > default
     * experimentConfigs = tbconfig:experimentBenchmarkEnvironment > command line > tbconfig > default
     *
     * This functions handles the tsconfig:** part since it is assumed that parent function that passed input "flags"
     * would've handled "command line > tbconfig > default"
     *
     * @param this.parsedConfig - Object containing configs parsed from the Command class
     */
    private generateControlExperimentServerConfig;
}
//# sourceMappingURL=index.d.ts.map